'**************************************
'*  mocog v0.2                        *
'*  An emulation of the Motorola 6809 *
'*  (C) 2009 Michael Rychlik          *
'**************************************
'
'v0.0 -  11 May 2009 - Original version
'
'v0.1 -  28 May 2009 - First signs of working, NOP, ADDA $00, JMP $0000 !!!
'
'v0.2 -  15 Jun 2009 - Most arithmentic and logic ops working (No flag settings implemented yet)
'                      Immediate, Direct, Extended addressing modes working.
'
CON
'==========================================================================================================
'THE FOLLOWING IS REQUIRED FOR THE HOMESPUN V024 COMPILER
'Uncomment one of the following CPU emulation types.

'Uncomment one of the following platform types.
'#define TriBladeProp
'#define PropDemoBoard

'#define banked_memory                                  'Uncomment if banked memory is to emulated.
'#define SingleStep                                     'Uncomment if "single step" emulation required.
'#define HEARTBEAT                                      'Uncomment if emulator is to flash a heartbeat LED
'==========================================================================================================


#ifdef TriBladeProp
#elseifdef PropDemoBoard
#else
#warn No HW platform type defined
#endif


  '6809 flag bits
  entire_flag     = %10000000
  fast_int_mask   = %01000000
  half_carry_flag = %00100000
  interrupt_mask  = %00010000
  negative_flag   = %00001000
  zero_flag       = %00000100
  overflow_flag   = %00000010
  carry_flag      = %00000001


  'I/O control block commands
  io_cmd_out   = $01
  io_cmd_in    = $02
  io_cmd_break = $03

#ifdef banked_memory
  'Memory bank select port.
  bank_sel_port = $40
#endif


VAR
  long  cog


PUB start(cpu_params) : okay | reg_base, io_base

'' Start CPU simulator - starts a cog
'' returns false if no cog available
''
''   cpu_par_params is a pointer to CPU parameter list

  stop

  reg_base := LONG[cpu_params]
  b_reg    := reg_base + 0
  a_reg    := reg_base + 1
  d_reg    := b_reg
  x_reg    := reg_base + 2
  y_reg    := reg_base + 4
  u_reg    := reg_base + 6
  s_reg    := reg_base + 8
  pc_reg   := reg_base + 10
  dp_reg   := reg_base + 13
  cc_reg   := reg_base + 12

#ifdef TriBladeProp
#elseifdef PropDemoBoard
  ram_base    := LONG[cpu_params + 4]
  ram_size    := LONG[cpu_params + 8]
  rom_base    := LONG[cpu_params + 12]
  rom_address := LONG[cpu_params + 16]
#endif

  io_base    := LONG[cpu_params + 20]
  io_command := io_base
  io_port    := io_base + 4
  io_data    := io_base + 8

  dispatch_tab_addr     := @dispatch_table_0
  dispatch_tab_0_addr   := @dispatch_table_0
  dispatch_tab_1_addr   := @dispatch_table_1
  dispatch_tab_2_addr   := @dispatch_table_2

  overlay_tab_addr       := @overlay_table

  'The following code sets up the Overlay Load Parameters
  O_00_ovl_     := OverlayParams(@O_00_ovl,    @O_00_ovl_end)
  O_01_ovl_     := OverlayParams(@O_01_ovl,     @O_01_ovl_end)
  O_02_ovl_     := OverlayParams(@O_02_ovl,     @O_02_ovl_end)
  O_03_ovl_     := OverlayParams(@O_03_ovl,     @O_03_ovl_end)
  O_04_ovl_     := OverlayParams(@O_04_ovl,    @O_04_ovl_end)
  O_05_ovl_     := OverlayParams(@O_05_ovl,     @O_05_ovl_end)
  O_06_ovl_     := OverlayParams(@O_06_ovl,     @O_06_ovl_end)
  O_07_ovl_     := OverlayParams(@O_07_ovl,     @O_07_ovl_end)
  O_08_ovl_    := OverlayParams(@O_08_ovl,    @O_08_ovl_end)
  O_09_ovl_    := OverlayParams(@O_09_ovl,    @O_09_ovl_end)
  O_0A_ovl_    := OverlayParams(@O_0A_ovl,    @O_0A_ovl_end)
  O_0B_ovl_    := OverlayParams(@O_0B_ovl,    @O_0B_ovl_end)
  O_0C_ovl_    := OverlayParams(@O_0C_ovl,    @O_0C_ovl_end)
  O_0D_ovl_    := OverlayParams(@O_0D_ovl,    @O_0D_ovl_end)
  O_0E_ovl_    := OverlayParams(@O_0E_ovl,    @O_0E_ovl_end)
  O_0F_ovl_    := OverlayParams(@O_0F_ovl,    @O_0F_ovl_end)
  O_10_ovl_    := OverlayParams(@O_10_ovl,    @O_10_ovl_end)
  out_ovl_     := OverlayParams(@out_ovl,     @out_ovl_end)
  in_ovl_      := OverlayParams(@in_ovl,      @in_ovl_end)
  O_13_ovl_ := OverlayParams(@O_13_ovl, @O_13_ovl_end)
  O_14_ovl_ := OverlayParams(@O_14_ovl, @O_14_ovl_end)
  O_15_ovl_ := OverlayParams(@O_15_ovl, @O_15_ovl_end)
  O_16_ovl_ := OverlayParams(@O_16_ovl, @O_16_ovl_end)
  O_17_ovl_ := OverlayParams(@O_17_ovl, @O_17_ovl_end)
  O_18_ovl_ := OverlayParams(@O_18_ovl, @O_18_ovl_end)
  O_19_ovl_ := OverlayParams(@O_19_ovl, @O_19_ovl_end)
  break_ovl_   := OverlayParams(@break_ovl,   @break_ovl_end)
  '....... add more overlays as required.

  'Initilize registers
  pc := WORD[pc_reg]
  cc := BYTE[cc_reg]
  sp := WORD[s_reg]

  okay := cog := cognew(@entry, 0) + 1                     'Start emulator in a new COG

PUB OverlayParams (o_start, o_end) : params | len, hubend, cogend
'This code sets up the parameters for an overlay (in a format to increase overlay loading speed)
  len := o_end - o_start                                   'length of hub overlay (bytes)
  hubend := o_start + len - 1                              'hub END address (last long) +3 (speed quirk in overlay loader)
  cogend := ((@OVERLAY_START - @entry + len) / 4) - 1      'cog END address (last long)
  params := hubend << 16 + cogend                          'set parameters


PUB stop

'' Stop CPU simulator - frees a cog

  if cog
    cogstop(cog~ - 1)


DAT
'---------------------------------------------------------------------------------------------------------

                        org     0

'---------------------------------------------------------------------------------------------------------
entry
OVERLAY_LOAD_RET
                        jmp       #RESIDENT_CODE        'Go execute resident code first (loaded automatically at CogNew)
                                                        'Can use as a RET instruction AFTER execution of the loaded overlay
'---------------------------------------------------------------------------------------------------------
'RESIDENT ROUTINES follow...
'
RESIDENT_CODE
#ifdef TriBladeProp
'the following are also used as variables later...
'The TriBladeProp Blade #2 driver presets the latch to enable SRAM -CE(U23)=0 and hold LE=0
datax                   mov     outa, ram_latched       ' -WE=1 -OE=1, all other bits=0
outx                    mov     dira, ram_dir_read      ' setup direction to read
#endif

#ifdef HEARTBEAT
                        mov     dira, heartbeat_pin     'Set Pins to output
#endif
                        jmp     #fetch
'---------------------------------------------------------------------------------------------------------

'Instruction fetch loop starts here
disable_interrupts
enable_interrupts
fetch
#ifdef SingleStep
'The following 2 lines do 6809 single stepping
                        mov     OVERLAY_PAR, break_ovl_    '\\ copy break_ovl's parameters
                        call    #OVERLAY_LOAD              '// load & execute break_ovl
#endif

#ifdef HEARTBEAT
                        xor     outa, heartbeat_pin        'N.B. If using heartbeat for KIPS measurement                                                           'remember to multiply the frequency by 2.
#endif
                        mov     dispatch_tab_addr, dispatch_tab_0_addr

dispatch                mov     address, pc
                        call    #read_memory_byte          'Read the instruction byte there
                        add     pc, #1                     'Increment the program counter

dispatch_2              shl     data_8, #2                 'Mul by 4 to get a LONG offset into the table
                        add     data_8, dispatch_tab_addr  'Index into table
                        rdlong  vector, data_8             'Get vector from table
vect_1                  jmp     vector                     'Jump(indirect) to the instruction handler

vect_2
vect_3                  shr     vector, #9                 'shift to 2nd (or 3rd) vector
                        jmp     vector                     'Jump(indirect) to the instruction handler


pc                      long    0                          '6809's Pogram Conter
sp                      long    0                          '6809's Stack pointer
cc                      long    0                          '6809's flags

#ifdef HEARTBEAT
heartbeat_pin           long    %00000000_00000001_00000000_00000000
#endif

dispatch_tab_addr       long    0
dispatch_tab_0_addr     long    0-0
dispatch_tab_1_addr     long    0-0
dispatch_tab_2_addr     long    0-0

vector                  long    0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'Data Source Functions

get_byte_a              rdbyte  data_8, a_reg
                        jmp     #vect_2

get_byte_b              rdbyte  data_8, b_reg
                        jmp     #vect_2

get_word_d              rdword  data_16, d_reg
                        jmp     #vect_2

get_word_x              rdword  data_16, x_reg
                        jmp     #vect_2

get_word_y              rdword  data_16, y_reg
                        jmp     #vect_2

get_word_s              rdword  data_16, s_reg
                        jmp     #vect_2

get_word_u              rdbyte  data_16, u_reg
                        jmp     #vect_2

get_pc                  mov     data_16, pc
                        jmp     #vect_2

get_byte_direct         mov     address, pc                'Get LOW direct address byte from immediate
                        call    #read_memory_byte
                        add     pc, #1
{FIXME DP should be in COG}
                        rdbyte  address, dp_reg            'Get HIGH direct address byte fro direct page reg
                        shl     address, #8
                        add     address, data_8            'Combine to get direct address
                        call    #read_memory_byte          'Get that byte operand
                        jmp     #vect_2




get_byte_indexed

get_byte_extended       mov     address, pc                'Read a WORD from location given by "pc"
                        call    #read_memory_word          'of 6809 RAM into byte data_8
                        add     pc, #2                     'and increment pc.
                        mov     address, data_16           'Use that as effective address of required byte
                        call    #read_memory_byte
                        jmp     #vect_2



get_byte_immediate      mov     address, pc                'Read a BYTE from location given by "pc"
                        call    #read_memory_byte          'of 6809 RAM into byte data_8
                        add     pc, #1                     'and increment pc.
                        jmp     #vect_2

get_word_immediate      mov     address, pc                'Read a WORD from location given by "pc"
                        call    #read_memory_word          'of 680 RAM into byte data_16 and pc = pc + 2
                        add     pc, #2
                        jmp     #vect_2

get_word_direct         mov     address, pc                'Get LOW direct address byte from immediate
                        call    #read_memory_byte
                        add     pc, #1
{FIXME DP should be in COG}
                        rdbyte  address, dp_reg            'Get HIGH direct address byte fro direct page reg
                        shl     address, #8
                        add     address, data_8            'Combine to get direct address
                        call    #read_memory_word          'Get that WORD operand
                        jmp     #vect_2


get_word_indexed


'00 = X
'01 = Y
'10 = U
'11 = S

increment               LONG    0
effective_address       LONG    0
index_reg               LONG    0                          'Pointer reg used for indexed addressing.
post_byte               LONG    0

ea_done

check_indirection       test    post_byte, #$10 wz         'Check the indirection bit in the post_byte
              if_z      jmp     #ea_done
                        mov     address, effective_address 'Indirection required
                        call    #read_memory_word
                        mov     effective_address, address
                        jmp     #ea_done

',R+
index_post_inc_1        mov     increment, #1
index_post_inc          rdword  effective_address, index_reg
                        mov     address, effective_address
                        add     address, increment
                        wrword  address, index_reg
                        jmp     #check_indirection

',R++
index_post_inc_2        mov     increment, #2
                        jmp     #index_post_inc

',-R
index_pre_dec_1         mov     increment, #1
index_pre_dec           rdword  effective_address, index_reg
                        sub     effective_address, #increment
                        wrword  effective_address, index_reg
                        jmp     #check_indirection

',--R
index_pre_dec_2         mov     increment, #2
                        jmp     #index_pre_dec


',R
index                   rdword  effective_address, index_reg
                        jmp     #check_indirection


'(+/- B),R
index_b                 rdbyte  data_8, b_reg              'Get the offset from B register
offset_8                rdword  effective_address, index_reg
offset_8_tail           shl     data_8, #24                'Offset is two's comp. so sign extend it.
                        sar     data_8, #24
                        add     effective_address, data_8  'Add the offset.
                        jmp     #check_indirection

'(+/- A),R
index_a                 rdbyte  data_8, a_reg              'Get the offset from A register
                        jmp     #offset_8


'(+/- 7 bit offset),R
index_offset_8          mov     address, pc                'Get offset from immediate byte to data_8
                        call    #read_memory_byte
                        add     pc, #1
                        jmp     #offset_8

'(+/- 15 bit offset),R
index_offset_16         mov     address, pc                'Get offset from imediate word to data_16
                        call    #read_memory_word
                        add     pc, #2
offset_16               rdword  effective_address, index_reg
offset_16_tail          shl     data_16, #16               'Offset is two's comp. so sign extend it.
                        sar     data_16, #16
                        add     effective_address, data_16 'Add the offset.
                        jmp     #check_indirection

'(+/- D),R
index_d                 rdword  data_16, d_reg             'Get the offset register
                        jmp     #offset_16

'(+/- 7 bit offset),PC
pc_offset_8             mov     address, pc                'Get target displacement from immediate byte
                        call    #read_memory_byte
                        add     pc, #1
                        mov     effective_address, pc
                        jmp     #offset_8_tail


'(+/- 15 bit offset),PC
pc_offset_16            mov     address, pc                'Get target displacement from immediate word
                        call    #read_memory_word
                        add     pc, #2
                        mov     effective_address, pc
                        jmp     #offset_16_tail

'[address]
extended_indirect       mov     address, pc                'Get target displacement from immediate word
                        call    #read_memory_word
                        add     pc, #2
                        mov     effective_address, data_16
                        jmp     #check_indirection         'FIXME Should check for lack of indirection here!!






get_word_extended

get_ea_direct

get_ea_indexed

get_ea_extended         mov     address, pc                'Get target displacement from immediate word
                        call    #read_memory_word
                        add     pc, #2
                        mov     effective_address, data_16
                        jmp     #vect_2


'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'Data Destination Functions

put_byte_a              wrbyte  data_8, a_reg
                        jmp     #fetch

put_byte_b              wrbyte  data_8, b_reg
                        jmp     #fetch

put_d_direct            mov     alu_reg, d_reg
put_word_direct         rdword  alu, alu_reg
                        mov     address, pc                'Get LOW direct address byte from immediate
                        call    #read_memory_byte
                        add     pc, #1
{FIXME DP should be in COG}
                        rdbyte  address, dp_reg            'Get HIGH direct address byte fro direct page reg
                        shl     address, #8
                        add     address, data_8            'Combine to get direct address
                        mov     data_16, alu
                        call    #write_memory_word         'Put that WORD
                        jmp     #fetch

put_x_direct            mov     alu_reg, x_reg
                        jmp     #put_word_direct

put_y_direct            mov     alu_reg, y_reg
                        jmp     #put_word_direct

put_s_direct            mov     alu_reg, s_reg
                        jmp     #put_word_direct

put_u_direct            mov     alu_reg, u_reg
                        jmp     #put_word_direct


put_word_d              wrword  data_16, d_reg
                        jmp     #fetch

put_word_x              wrword  data_16, x_reg
                        jmp     #fetch

put_word_y              wrword  data_16, y_reg
                        jmp     #fetch

put_word_u              wrword  data_16, u_reg
                        jmp     #fetch

put_word_s              wrword  data_16, s_reg
                        jmp     #fetch


put_a_direct            mov     alu_reg, a_reg
put_byte_direct         rdbyte  alu, alu_reg
                        mov     address, pc                'Get LOW direct address byte from immediate
                        call    #read_memory_byte
                        add     pc, #1
{FIXME DP should be in COG}
                        rdbyte  address, dp_reg            'Get HIGH direct address byte fro direct page reg
                        shl     address, #8
                        add     address, data_8            'Combine to get direct address
                        mov     data_8, alu
                        call    #write_memory_byte         'Put that byte
                        jmp     #fetch

put_b_direct            mov     alu_reg, b_reg
                        jmp     #put_byte_direct

put_byte_indexed

put_word_indexed

put_byte_extended

put_word_extended


put_byte_ea             call    #write_memory_byte
                        jmp     #fetch



'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'Memory Access Functions

#ifdef TriBladeProp
'                                          +-------- -CS (U26 FLASH)   -+
'                                          |+------- -CS (J22 microSD)  +- Latched pins...
'                                          ||+------ -RST3              |    passes when LE=1
'             (P30) SO ---+                |||+----- -RST1              |    latches on  LE low going edge
'             (P31) SI --+|                ||||+---- -CE (U24 SRAM-hi)  |    latched on  LE=0
'                        ||+---- -WE(SDA)  |||||+--- -CE (U23 SRAM-lo)  |
'                        |||+--- -OE(SCL)  ||||||+-- A20                |
'                        ||||+--  LE       |||||||+- A19               -+
'                        |||||             ||||||||
'          P31...P0 --> %00xx0_xxxxxxxxxxx_xxxxxxxx_xxxxxxxx
ram_latched     long    %00110_00000000000_00000000_00000000 ' -WE=1 -OE=1 (LE=0)
ram_read        long    %00100_00000000000_00000000_00000000 ' -WE=1 -OE=0 (LE=0)
ram_write       long    %00010_00000000000_00000000_00000000 ' -WE=0 -OE=1 (LE=0)
ram_dir_read    long    %00110_11111111111_11111111_00000000 ' outputs WE, OE, A0-18, inputs D0-7
ram_dir_write   long    %00110_11111111111_11111111_11111111 ' outputs WE, OE, A0-18, D0-7
ram_dir_input   long    %00000_00000000000_00000000_00000000 ' all inputs (for Ram Disk access)
'                              +---------+ +------+ +------+
'                              A18......A8 A7....A0 D7....D0
'                                          Q7....Q0
'
'outx            long    0                                   'used as a temp address + WE/OE bits

'SRAM Initialisation: TriBladeProp Blade #2 Driver presets latch to -CE(U23)

'Read a BYTE from location given by "address" of 6809 memory space into byte data_8
read_memory_byte
#ifdef banked_memory
                        cmp     address, common_ram wc     'Address in resident RAM area ?
              if_c      add     address, bank_offset       'No: Add offset to current memory bank in ext RAM
#endif
                        mov     outx, address
                        shl     outx,#8                    'shift address 8 bits
                        or      outx, ram_read             'add -WE=1 -OE=0
                        mov     outa, outx                 'pc (shifted 8 bits + -WE=1 -OE=0)
                        nop                                'delay for sram access time
                        mov     data_8, ina                'read SRAM
                        and     data_8, #$FF               'extract 8 bits
read_memory_byte_ret    ret

write_memory_byte
#ifdef banked_memory
                        and     address, mask_low_word     'Ensure address is within 64K
                        cmp     address, common_ram wc     'Address in resident RAM area ?
              if_c      add     address, bank_offset       'No: Add offset to current memory bank
#endif
              '''       cmp     address, ram_size wc,wz    'Is the given address within the RAM range ?
              {if_b}    mov     outx, address
              {if_b}    shl     outx, #8                   'shift address 8 bits
              {if_b}    or      outx, ram_write            'add -WE=0 -OE=1
              {if_b}    mov     datax, data_8
              {if_b}    and     datax, #$FF                'ensure upper bytes=0
              {if_b}    or      outx, datax                'add data bits to be written
              {if_b}    mov     outa, outx                 'output -WE=0 -OE=1, address bits, data bits
              {if_b}    mov     dira, ram_dir_write        'set data bits to outputs
'             {if_b}    nop                                '25nS min delay (nothing reqd)
              {if_b}    or      outa, ram_latched          'set -WE=0->1 -OE=1->1  (end write)
              {if_b}    mov     dira, ram_dir_read         'set data bits to inputs
              {if_b}    mov     outa, ram_read             'set -WE=1 -OE=0
write_memory_byte_ret   ret

#ifdef banked_memory
common_ram              long    $C000                      'Address of common (resident) RAM in 6809 space.
bank_offset             long    0                          'Offset of the current memory bank in external RAM
#endif

#elseifdef PropDemoBoard

'Read a BYTE from location given by "address" of 6809 memory space into byte data_8
read_memory_byte        cmp     address, ram_size wc,wz    'Is the given address within the RAM range ?
              if_ae     jmp     #read_rom
                        mov     hub_pointer, ram_base      'Point to the RAM
                        add     hub_pointer, address       'Add address offset
                        rdbyte  data_8, hub_pointer        'Read the byte from there
read_memory_byte_ret    ret

read_rom                mov     rom_address, address
                        sub     rom_address, rom_start     'address becomes offset into ROM from start of ROM
                        mov     hub_pointer, rom_base      'Point to the ROM
                        add     hub_pointer, rom_address   'Add address offset
                        rdbyte  data_8, hub_pointer        'Read the byte from there
                        jmp     read_memory_byte_ret       'return(indirect)

'Write the BYTE data_8 to location given by "address"
write_memory_byte       cmp     address, ram_size wc,wz    'Is the given address within the RAM range ?
              if_b      mov     hub_pointer, ram_base      'Point to the RAM
              if_b      add     hub_pointer, address       'Add address offset
              if_b      wrbyte  data_8, hub_pointer        'Write the byte there
write_memory_byte_ret   ret

'The following configuration "constants" set from COG PARameters.
ram_base                long    0-0                        'HUB address of the 6809 RAM area
ram_size                long    0-0                        'Length in bytes of available 6809 RAM
rom_base                long    0-0                        'HUB address of the BOOT ROM image
rom_start               long    0-0                        'ROM starts at this address in 6809 space
rom_address             long    0-0

hub_pointer             long    ram_base                   'Pointer into HUB RAM

#endif

'Read a WORD from location given by "address" of 6809 memory space into byte data_16
read_memory_word        call    #read_memory_byte          'Read the high byte
                        mov     data_16, data_8
                        shl     data_16, #8
                        add     address, #1
                        call    #read_memory_byte          'Read the low byte
                        or      data_16, data_8
read_memory_word_ret    ret

'Write the WORD data_16 to location given by "address"
write_memory_word       mov     data_8, data_16
                        call    #write_memory_byte         'Write the LOW byte
                        shr     data_8, #8
                        add     address, #1
                        call    #write_memory_byte
write_memory_word_ret   ret


'6809 RAM read/write vars.
address                 long    0                          'Address param for read/write funcs
data_8                  long    0                          'Data bytes read into here
data_16                 long    0                          'Data words read into here
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'6809 Operation Functions

alu                     LONG    0                          'Arithmetic Logic Unit accumulator
flags                   LONG    0                          'The 6809 flag register working copy

neg_byte                neg     data_8, data_8
                        jmp     #vect_3

com_byte                xor     data_8, #$FF
                        jmp     #vect_3


lsr_byte                shr     data_8, #1 wc
                        muxc    cc, #carry_flag
                        jmp     #vect_3


ror_byte                test    cc, #carry_flag wz         'Set operands bit 8 from carry flag
                        muxnz   data_8, #%100000000
                        shr     data_8, #1 wc
                        muxc    cc, #carry_flag
                        jmp     #vect_3

lsl_byte                shl     data_8, #1
                        test    data_8, #%100000000 wz
                        muxnz   cc, #carry_flag
                        jmp     #vect_3

rol_byte                shl     data_8, #1
                        test    cc, #carry_flag wz         'Set bit 1 from carry flag
                        muxnz   data_8, #%00000001
                        test    data_8, #%100000000 wz
                        muxnz   cc, #carry_flag
                        jmp     #vect_3

asr_byte                shl     data_8, #24
                        sar     data_8, #25 wc
                        muxc    cc, #carry_flag
                        jmp     #vect_3

dec_byte                sub     data_8, #1
                        jmp     #vect_3

inc_byte                add     data_8, #1
                        jmp     #vect_3

tst_byte                jmp     #vect_3

clr_byte                mov     data_8, #0
                        jmp     #vect_3


suba                    mov     alu_reg, a_reg
sub_tail                rdbyte  alu, alu_reg
                        sub     alu, data_8
                        wrbyte  alu, alu_reg
                        jmp     #fetch

subb                    mov     alu_reg, b_reg
                        jmp     #sub_tail

cmpa                    mov     alu_reg, a_reg
cmp_tail                rdbyte  alu, alu_reg
                        cmp     alu, data_8 wz, wc
                        jmp     #fetch

cmpb                    mov     alu_reg, b_reg
                        jmp     #cmp_tail


cmpx_                   rdword  alu, x_reg
cmpx_tail               sub     alu, data_16
                        jmp     #fetch

cmpy_                   rdword  alu, y_reg
                        jmp     #cmpx_tail

cmps_                   rdword  alu, s_reg
                        jmp     #cmpx_tail

cmpu_                   rdword  alu, u_reg
                        jmp     #cmpx_tail

cmpd_                   rdword  alu, d_reg
                        jmp     #cmpx_tail


sbca                    mov     alu_reg, a_reg
sbca_tail               rdbyte  alu, alu_reg
                        test    flags, #carry_flag wc      'Set Props carry from 6809 carry flag
                        subx    alu, data_8                'Sub with carry
                        wrbyte  alu, alu_reg
                        jmp     #fetch

sbcb                    mov     alu_reg, b_reg
                        jmp     #sbca_tail

anda                    mov     alu_reg, a_reg
anda_tail               rdbyte  alu, alu_reg
                        and     alu, data_8
                        wrbyte  alu, alu_reg
                        jmp     #fetch

andb                    mov     alu_reg, b_reg
                        jmp     #anda_tail

bita                    mov     alu_reg, a_reg
bita_tail               rdbyte  alu, alu_reg
                        test    alu, data_8 wc, wz
                        jmp     #fetch

bitb                    mov     alu_reg, b_reg
                        jmp     #bita_tail

eora                    mov     alu_reg, a_reg
eora_tail               rdbyte  alu, alu_reg
                        xor     alu, data_8
                        wrbyte  alu, alu_reg
                        jmp     #fetch

eorb                    mov     alu_reg, b_reg
                        jmp     #eora_tail


adca                    mov     alu_reg, a_reg
adca_tail               rdbyte  alu, alu_reg
                        test    flags, #carry_flag wc      'Set Props carry from 6809 carry flag
                        addx    alu, data_8                'Add with carry
                        wrbyte  alu, alu_reg
                        jmp     #fetch

adcb                    mov     alu_reg, b_reg
                        jmp     #adca_tail

ora                     mov     alu_reg, a_reg
ora_tail                rdbyte  alu, alu_reg
                        or      alu, data_8
                        wrbyte  alu, alu_reg
                        jmp     #fetch

orb                     mov     alu_reg, b_reg
                        jmp     #ora_tail

adda                    mov     alu_reg, a_reg
add_tail                rdbyte  alu, alu_reg
                        add     alu, data_8
                        wrbyte  alu, alu_reg
                        jmp     #fetch

addb                    mov     alu_reg, b_reg
                        jmp     #add_tail

addd                    rdword  alu, d_reg
                        add     alu, data_16
                        wrword  alu, d_reg
                        jmp     #fetch

subd                    rdword  alu, d_reg
                        sub     alu, data_16
                        wrword  alu, d_reg
                        jmp     #fetch


jmp_                    mov     pc, effective_address
                        jmp     #fetch


daa

orcc

andcc

'Everyones favorite instruction.
sex                     rdword  alu, d_reg                 'Read A:B regs as D
                        shl     alu, #24                   'Sign extend B into A.
                        sar     alu, #24
                        wrword  alu, d_reg                 'Put A:B back as reg D
                        jmp     #fetch

exg
tfr


not_implemented
illegal
halt
                        mov     OVERLAY_PAR, break_ovl_    '\\ copy break_ovl's parameters
                        call    #OVERLAY_LOAD              '// load & execute break_ovl
                        jmp     #fetch


vector_overlay          mov     ovl, vector                'Extract overlay param number from vector bits 2 to 26
                        shr     ovl, #7
                        and     ovl, #%11111100
                        add     ovl, overlay_tab_addr      'Offset into overlay parameter table
                        rdlong  OVERLAY_PAR, ovl           'Get this overlays parameters
                        jmp     #OVERLAY_LOAD              'load it and run it.

ovl                     long      0
overlay_tab_addr        long      0-0


'---------------------------------------------------------------------------------------------------------
'Misc variables

'---------------------------------------------------------------------------------------------------------
'Pointers to 6809 registers in HUB RAM
a_reg                   long    0
b_reg                   long    0
d_reg                   long    0
x_reg                   long    0
y_reg                   long    0
s_reg                   long    0
u_reg                   long    0
pc_reg                  long    0
dp_reg                  long    0
cc_reg                  long    0

alu_reg                 long    0                          'Can point to any register that
                                                           'is accumulator at the time
'---------------------------------------------------------------------------------------------------------
'The I/O control block
io_command              long    0
io_port                 long    0
io_data                 long    0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
' OVERLAY LOADER follows...
' An even number of longs will be loaded (for efficiency)
' Flags c & z will be maintained
'

Overlay_par             long    0-0                        'overlay parameters for the OVERLAY_LOAD

break_ovl_              long    0-0

_0x400                  long    $0000_0400                 'inc/decrement destination by 2
_djnz0                  djnz    overlay_par, #overlay_copy2 'prototype instruction (moved to overlay instruction)
t1                      long    0                          'used to determine if overlay already loaded (can be used by other code)

OVERLAY_LOAD            mov     OVERLAY_START, _djnz0      'Copy djnz instruction to head of overlay area.
                        movd    overlay_copy2, overlay_par 'move cog END address into rdlong instruction
                        sub     overlay_par, #1            'decrement cog End address by 1
                        movd    overlay_copy1, overlay_par 'move cog END-1 address into rdlong instruction
                        shr     overlay_par, #16           'extract the overlay## hub END address (remove cog address)
overlay_copy2           rdlong  0-0, overlay_par           'copy long from hub to cog   (hptr ignores last 2 bits!)
                        sub     overlay_par, #7            'decrement hub ptr by 1 long (prev by 1, now by 7)
                        sub     overlay_copy2, _0x400      'decrement cog (destination) address by 2
overlay_copy1           rdlong  0-0, overlay_par           'copy long from hub to cog
                        sub     overlay_copy1, _0x400      'decrement cog (destination) address by 2
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
OVERLAY_START '<--- djnz  overlay_par,#overlay_copy2    'decrement hub ptr by 1 long (now by 1, next by 7)
              '^^^ The above instruction is moved in by the loader and is overwritten by the first overlay
              '    instruction during the final overlay load (rdlong) instruction. (loading is done in reverse)
'---------------------------------------------------------------------------------------------------------
long    0[105]
                        fit       $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'daa overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
O_00_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
O_00_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'ldi overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
O_01_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
O_01_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'cpi overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
O_02_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
O_02_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'ini overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
O_03_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
O_03_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'outi overlay. This is a "load/execute" overlay.
                        org OVERLAY_START
O_04_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
O_04_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'ldd overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
O_05_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
O_05_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'cpd overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
O_06_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
O_06_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'ind overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
O_07_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
O_07_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'outd overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
O_08_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
O_08_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'ldir overlay. This is a "load/execute" overlay.
' (DE) <- (HL);
' HL   <- HL+1;
' DE   <- DE+1;
' BC   <- BC-1;
' repeat until BC = 0
                        org     OVERLAY_START
O_09_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
O_09_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'cpir overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
O_0A_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
O_0A_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'inir overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
O_0B_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
O_0B_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'otir overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
O_0C_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
O_0C_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'lddr overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
O_0D_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
O_0D_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'cpdr overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
O_0E_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
O_0E_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'indr overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
O_0F_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
O_0F_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'otdr overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
O_10_ovl
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
O_10_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'out overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
out_ovl                 mov     address, pc                'Get IO port no. from immediate byte
                        call    #read_memory_byte
                        add     pc, #1

#ifdef banked_memory
                        cmp     data_8, #bank_sel_port wz  'Is this a write to the bank select port ?
              if_nz     jmp     #:not_bank_sel

                        rdbyte  data_8, a_reg              'Get the output data from A reg
                        movs    :set_offset, #:bank_table
                        add     :set_offset, data_8        'Index into bank offset table
:set_offset             mov     bank_offset, 0-0           'Set the bank_offset from a table entry
                        jmp     #fetch

:bank_table
:bank_0_offset           long    $000000                    'Bank 0 is the 48K below common
:bank_1_offset           long    $010000                    'Bank 1 is the 48K above common, i.e. above 64K
:bank_2_offset           long    $01C000                    'Following banks are 48K blocks following on from Bank 1
:bank_3_offset           long    $028000
:bank_4_offset           long    $034000

:not_bank_sel
#endif
                        wrbyte  data_8, io_port
                        rdbyte  data_8, a_reg              'Get the output data from A reg
                        wrbyte  data_8, io_data            'Set the output data
                        mov     data_8, #io_cmd_out        'Set I/O command to OUT
#ifdef TriBladeProp
                        mov     dira, ram_dir_input        'RR20090321 (pass I/O to another cog)
#endif
                        wrbyte  data_8, io_command
:wait                   rdbyte  data_8, io_command wz      'Wait for the output done
              if_nz     jmp     #:wait
#ifdef TriBladeProp
                        mov     dira, ram_dir_read         'RR20090321 (take I/O from another cog)
#endif
                        jmp     #fetch

                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
out_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'in overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
in_ovl                  mov     address, pc                'Set IO port no. from immediate byte
                        call    #read_memory_byte
                        add     pc, #1
                        wrbyte  data_8, io_port
                        mov     data_8, #io_cmd_in         'Set I/O command to IN
#ifdef TriBladeProp
                        mov     dira, ram_dir_input        'RR20090321 (pass I/O to another cog)
#endif
                        wrbyte  data_8, io_command
:wait                   rdbyte  data_8, io_command wz      'Wait for the input data
              if_nz     jmp     #:wait
#ifdef TriBladeProp
                        mov     dira, ram_dir_read         'RR20090321 (take I/O from another cog)
#endif
                        rdbyte  data_8, io_data            'Read the input data to accumulator
                        wrbyte  data_8, a_reg
                        jmp     #fetch

                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
in_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'alu_neg overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
O_13_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
O_13_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'alu_sra overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
O_14_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
O_14_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'alu_srl overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
O_15_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
O_15_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'alu_srl overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
O_16_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
O_16_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'alu_bit overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
O_17_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
O_17_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'alu_res overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
O_18_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
O_18_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'alu_set overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
O_19_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
O_19_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'break overlay. This is a "called" overlay.
                        org     OVERLAY_START
break_ovl               wrword  pc, pc_reg
                        wrbyte  cc, cc_reg
                        'wrbyte  dp, dp_reg
                        'wrword  sp, s_reg
                        mov     data_8, #io_cmd_break      'Set I/O command to BREAK
#ifdef TriBladeProp
                        mov     dira, ram_dir_input        'RR20090321 (pass I/O to another cog)
#endif
                        wrbyte  data_8, io_command
:wait                   rdbyte  data_8, io_command wz
              if_nz     jmp     #:wait
#ifdef TriBladeProp
                        mov     dira, ram_dir_read         'RR20090321 (take I/O from another cog)
#endif
                        jmp     OVERLAY_LOAD_RET           'return(indirect) to calling routine

                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
break_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
CON
'Overlay numbers.
'Note: These overlay numbers must be the in the order of the overlay param
'      entries in the overlay_table.
O_00_ovl_no     = $00
O_01_ovl_no     = $01
O_02_ovl_no     = $02
O_03_ovl_no     = $03
O_04_ovl_no    = $04
O_05_ovl_no     = $05
O_06_ovl_no     = $06
O_07_ovl_no     = $07
O_08_ovl_no    = $08
O_09_ovl_no    = $09
O_0A_ovl_no    = $0A
O_0B_ovl_no    = $0B
O_0C_ovl_no    = $0C
O_0D_ovl_no    = $0D
O_0E_ovl_no    = $0E
O_0F_ovl_no    = $0F
O_10_ovl_no    = $10
out_ovl_no     = $11
in_ovl_no      = $12
O_13_ovl_no = $13
O_14_ovl_no = $14
O_15_ovl_no = $15
O_16_ovl_no = $16
O_17_ovl_no = $17
O_18_ovl_no = $18
O_19_ovl_no = $19
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
DAT
' Table of overlay parameters.
' Note: The overlay numbers defined in CON must be in the order
'       of the param entries here.
overlay_table
{00}O_00_ovl_            long    0-0                        '\\ parameters for overlays (preset by spin code)
{01}O_01_ovl_            long    0-0                        '||   bits 31..16 = hub END+3 address (must be even length)
{02}O_02_ovl_            long    0-0                        '||   bits  8..0  = cog END address
{03}O_03_ovl_            long    0-0                        '||
{04}O_04_ovl_           long    0-0                        '||
{05}O_05_ovl_            long    0-0                        '||
{06}O_06_ovl_            long    0-0                        '||
{07}O_07_ovl_            long    0-0                        '||
{08}O_08_ovl_           long    0-0                        '||
{09}O_09_ovl_           long    0-0                        '||
{0A}O_0A_ovl_           long    0-0                        '||
{0B}O_0B_ovl_           long    0-0                        '||
{0C}O_0C_ovl_           long    0-0                        '||
{0D}O_0D_ovl_           long    0-0                        '||
{0E}O_0E_ovl_           long    0-0                        '||
{0F}O_0F_ovl_           long    0-0                        '||
{10}O_10_ovl_           long    0-0                        '||
{11}out_ovl_            long    0-0                        '||
{12}in_ovl_             long    0-0                        '||
{13}O_13_ovl_        long    0-0                        '||
{14}O_14_ovl_        long    0-0                        '||
{15}O_15_ovl_        long    0-0                        '||
{16}O_16_ovl_        long    0-0                        '||
{17}O_17_ovl_        long    0-0                        '||
{18}O_18_ovl_        long    0-0                        '||
{19}O_19_ovl_        long    0-0                        '||

'---------------------------------------------------------------------------------------------------
dispatch_table_0
{00}	LONG	get_byte_direct    + (neg_byte << 9) + (put_byte_ea << 18)  'NEG       Direct    , 6,    2, uaaaa
{01}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{02}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{03}	LONG	get_byte_direct    + (com_byte << 9) + (put_byte_ea << 18)  'COM       Direct    , 6,    2, -aa01
{04}	LONG	get_byte_direct    + (lsr_byte << 9) + (put_byte_ea << 18)  'LSR       Direct    , 6,    2, -0a-s
{05}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{06}	LONG	get_byte_direct    + (ror_byte << 9) + (put_byte_ea << 18)  'ROR       Direct    , 6,    2, -aa-s
{07}	LONG	get_byte_direct    + (asr_byte << 9) + (put_byte_ea << 18)  'ASR       Direct    , 6,    2, uaa-s
{08}	LONG	get_byte_direct    + (lsl_byte << 9) + (put_byte_ea << 18)  'LSL/ASL   Direct    , 6,    2, naaas
{09}	LONG	get_byte_direct    + (rol_byte << 9) + (put_byte_ea << 18)  'ROL       Direct    , 6,    2, -aaas
{0A}	LONG	get_byte_direct    + (dec_byte << 9) + (put_byte_ea << 18)  'DEC       Direct    , 6,    2, -aaa-
{0B}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{0C}	LONG	get_byte_direct    + (inc_byte << 9) + (put_byte_ea << 18)  'INC       Direct    , 6,    2, -aaa-
{0D}	LONG	get_byte_direct    + (tst_byte << 9)                        'TST       Direct    , 6,    2, -aa0-
{0E}	LONG	get_ea_direct      + (jmp_ << 9)                            'JMP       Direct    , 3,    2, -----
{0F}	LONG	get_byte_direct    + (clr_byte << 9) + (put_byte_ea << 18)  'CLR       Direct    , 6,    2, -0100

{10}	LONG	0	                                                    'PAGE1+    Variant   , 1,    1, +++++
{11}	LONG	0	                                                    'PAGE2+    Variant   , 1,    1, +++++
{12}	LONG	fetch                                                       'NOP       Inherent  , 2,    1, -----
{13}	LONG	not_implemented	                                            'SYNC      Inherent  , 2,    1, -----
{14}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{15}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{16}	LONG	0	'LBRA      Relative  , 5,    3, -----
{17}	LONG	0	'LBSR      Relative  , 9,    3, -----
{18}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{19}	LONG	daa	                                                    'DAA       Inherent  , 2,    1, -aa0a
{1A}	LONG	get_byte_immediate + (orcc << 9)                            'ORCC      Immediate , 3,    2, ddddd
{1B}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{1C}	LONG	get_byte_immediate + (andcc << 9)                           'ANDCC     Immediate , 3,    2, ddddd
{1D}	LONG	sex                	                                    'SEX       Inherent  , 2,    1, -aa0-
{1E}	LONG	get_byte_immediate + (exg << 9)	                            'EXG       Immediate  , 8,    2, ccccc
{1F}	LONG	get_byte_immediate + (tfr << 9)	                            'TFR       Inherent  , 7,    2, ccccc

{20}	LONG	0	'BRA       Relative  , 3,    2, -----
{21}	LONG	0	'BRN       Relative  , 3,    2, -----
{22}	LONG	0	'BHI       Relative  , 3,    2, -----
{23}	LONG	0	'BLS       Relative  , 3,    2, -----
{24}	LONG	0	'BHS/BCC   Relative  , 3,    2, -----
{25}	LONG	0	'BLO/BCS   Relative  , 3,    2, -----
{26}	LONG	0	'BNE       Relative  , 3,    2, -----
{27}	LONG	0	'BEQ       Relative  , 3,    2, -----
{28}	LONG	0	'BVC       Relative  , 3,    2, -----
{29}	LONG	0	'BVS       Relative  , 3,    2, -----
{2A}	LONG	0	'BPL       Relative  , 3,    2, -----
{2B}	LONG	0	'BMI       Relative  , 3,    2, -----
{2C}	LONG	0	'BGE       Relative  , 3,    2, -----
{2D}	LONG	0	'BLT       Relative  , 3,    2, -----
{2E}	LONG	0	'BGT       Relative  , 3,    2, -----
{2F}	LONG	0	'BLE       Relative  , 3,    2, -----

{30}	LONG	get_ea_indexed     + (put_word_x)   	                    'LEAX      Indexed   , 4,    2, --a--
{31}	LONG	get_ea_indexed     + (put_word_y)	                    'LEAY      Indexed   , 4,    2, --a--
{32}	LONG	get_ea_indexed     + (put_word_s)	                    'LEAS      Indexed   , 4,    2, -----
{33}	LONG	get_ea_indexed     + (put_word_u)                           'LEAU      Indexed   , 4,    2, -----
{34}	LONG	0	'PSHS      Inherent  , 5,    2, -----
{35}	LONG	0	'PULS      Inherent  , 5,    2, ccccc
{36}	LONG	0	'PSHU      Inherent  , 5,    2, -----
{37}	LONG	0	'PULU      Inherent  , 5,    2, ccccc
{38}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{39}	LONG	0	'RTS       Inherent  , 5,    1, -----
{3A}	LONG	0	'ABX       Inherent  , 3,    1, -----
{3B}	LONG	0	'RTI       Inherent  , 6/15, 1, -----
{3C}	LONG	0	'CWAI      Inherent  , 21,   2, ddddd
{3D}	LONG	0	'MUL       Inherent  , 11,   1, --a-a
{3E}	LONG	0	'RESET*    Inherent  , *,    1, *****
{3F}	LONG	0	'SWI       Inherent  ,  19,  1, -----

{40}	LONG	get_byte_a         + (neg_byte << 9) + (put_byte_a << 18)   '*NEGA      Inherent  , 2,    1, uaaaa
{41}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{42}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{43}	LONG	get_byte_a         + (com_byte << 9) + (put_byte_a << 18)   '*COMA      Inherent  , 2,    1, -aa01
{44}	LONG	get_byte_a         + (lsr_byte << 9) + (put_byte_a << 18)   '*LSRA      Inherent  , 2,    1, -0a-s
{45}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{46}	LONG	get_byte_a         + (ror_byte << 9) + (put_byte_a << 18)   '*RORA      Inherent  , 2,    1, -aa-s
{47}	LONG	get_byte_a         + (asr_byte << 9) + (put_byte_a << 18)   '*ASRA      Inherent  , 2,    1, uaa-s
{48}	LONG	get_byte_a         + (lsl_byte << 9) + (put_byte_a << 18)   '*LSLA/ASLA Inherent  , 2,    1, naaas
{49}	LONG	get_byte_a         + (rol_byte << 9) + (put_byte_a << 18)   '*ROLA      Inherent  , 2,    1, -aaas
{4A}	LONG	get_byte_a         + (dec_byte << 9) + (put_byte_a << 18)   '*DECA      Inherent  , 2,    1, -aaa-
{4B}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{4C}	LONG	get_byte_a         + (inc_byte << 9) + (put_byte_a << 18)   '*INCA      Inherent  , 2,    1, -aaa-
{4D}	LONG	get_byte_a         + (tst_byte << 9)                        'TSTA      Inherent  , 2,    1, -aa0-
{4E}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{4F}	LONG	get_byte_a         + (clr_byte << 9) + (put_byte_a << 18)   '*CLRA      Inherent  , 2,    1, -0100

{50}	LONG    get_byte_b         + (neg_byte << 9) + (put_byte_b << 18)   '*NEGB      Inherent  , 2,    1, uaaaa
{51}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{52}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{53}	LONG	get_byte_b         + (com_byte << 9) + (put_byte_b << 18)   '*COMB      Inherent  , 2,    1, -aa01
{54}	LONG	get_byte_b         + (lsr_byte << 9) + (put_byte_b << 18)   '*LSRB      Inherent  , 2,    1, -0a-s
{55}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{56}	LONG	get_byte_b         + (ror_byte << 9) + (put_byte_b << 18)   '*RORB      Inherent  , 2,    1, -aa-s
{57}	LONG	get_byte_b         + (asr_byte << 9) + (put_byte_b << 18)   '*ASRB      Inherent  , 2,    1, uaa-s
{58}	LONG	get_byte_b         + (lsl_byte << 9) + (put_byte_b << 18)   '*LSLB/ASLB Inherent  , 2,    1, naaas
{59}	LONG	get_byte_b         + (rol_byte << 9) + (put_byte_b << 18)   '*ROLB      Inherent  , 2,    1, -aaas
{5A}	LONG	get_byte_b         + (dec_byte << 9) + (put_byte_b << 18)   '*DECB      Inherent  , 2,    1, -aaa-
{5B}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{5C}	LONG	get_byte_b         + (inc_byte << 9) + (put_byte_b << 18)   '*INCB      Inherent  , 2,    1, -aaa-
{5D}	LONG    get_byte_b         + (tst_byte << 9)                        'TSTB      Inherent  , 2,    1, -aa0-
{5E}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{5F}	LONG	get_byte_b         + (clr_byte << 9) + (put_byte_b << 18)   '*CLRB      Inherent  , 2,    1, -0100

{60}	LONG	get_byte_indexed   + (neg_byte << 9) + (put_byte_ea << 18)  'NEG       Indexed   , 6,    2, uaaaa
{61}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{62}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{63}	LONG	get_byte_indexed   + (com_byte << 9) + (put_byte_ea << 18)  'COM       Indexed   , 6,    2, -aa01
{64}	LONG	get_byte_indexed   + (lsr_byte << 9) + (put_byte_ea << 18)  'LSR       Indexed   , 6,    2, -0a-s
{65}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{66}	LONG	get_byte_indexed   + (ror_byte << 9) + (put_byte_ea << 18)  'ROR       Indexed   , 6,    2, -aa-s
{67}	LONG	get_byte_indexed   + (asr_byte << 9) + (put_byte_ea << 18)  'ASR       Indexed   , 6,    2, uaa-s
{68}	LONG	get_byte_indexed   + (lsl_byte << 9) + (put_byte_ea << 18)  'LSL/ASL   Indexed   , 6,    2, naaas
{69}	LONG	get_byte_indexed   + (rol_byte << 9) + (put_byte_ea << 18)  'ROL       Indexed   , 6,    2, -aaas
{6A}	LONG	get_byte_indexed   + (dec_byte << 9) + (put_byte_ea << 18)  'DEC       Indexed   , 6,    2, -aaa-
{6B}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{6C}	LONG	get_byte_indexed   + (inc_byte << 9) + (put_byte_ea << 18)  'INC       Indexed   , 6,    2, -aaa-
{6D}	LONG	get_byte_indexed   + (tst_byte << 9)                        'TST       Indexed   , 6,    2, -aa0-
{6E}	LONG	get_ea_indexed     + (jmp_ << 9)                            'JMP       Indexed   , 3,    2, -----
{6F}	LONG	get_byte_indexed   + (clr_byte << 9) + (put_byte_ea << 18)  'CLR       Indexed   , 6,    2, -0100

{70}	LONG	get_byte_extended  + (neg_byte << 9) + (put_byte_ea << 18)  'NEG       Extended  , 7,    3, uaaaa
{71}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{72}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{73}	LONG	get_byte_extended  + (com_byte << 9) + (put_byte_ea << 18)  'COM       Extended  , 7,    3, -aa01
{74}	LONG	get_byte_extended  + (lsr_byte << 9) + (put_byte_ea << 18)  'LSR       Extended  , 7,    3, -0a-s
{75}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{76}	LONG	get_byte_extended  + (ror_byte << 9) + (put_byte_ea << 18)  'ROR       Extended  , 7,    3, -aa-s
{77}	LONG	get_byte_extended  + (asr_byte << 9) + (put_byte_ea << 18)  'ASR       Extended  , 7,    3, uaa-s
{78}	LONG	get_byte_extended  + (lsl_byte << 9) + (put_byte_ea << 18)  'LSL/ASL   Extended  , 7,    3, naaas
{79}	LONG	get_byte_extended  + (rol_byte << 9) + (put_byte_ea << 18)  'ROL       Extended  , 7,    3, -aaas
{7A}	LONG	get_byte_extended  + (dec_byte << 9) + (put_byte_ea << 18)  'DEC       Extended  , 7,    3, -aaa-
{7B}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{7C}	LONG	get_byte_extended  + (inc_byte << 9) + (put_byte_ea << 18)  'INC       Extended  , 7,    3, -aaa-
{7D}	LONG	get_byte_extended  + (tst_byte << 9)                        'TST       Extended  , 7,    3, -aa0-
{7E}	LONG	get_ea_extended    + (jmp_ << 9)                            'JMP       Extended  , 3,    3, -----
{7F}	LONG	get_byte_extended  + (clr_byte << 9) + (put_byte_ea << 18)  'CLR       Extended  , 7,    3, -0100

{80}	LONG	get_byte_immediate + (suba << 9)                            '*SUBA      Immediate , 2,    2, uaaaa
{81}	LONG	get_byte_immediate + (cmpa << 9)                            'CMPA      Immediate , 2,    2, uaaaa
{82}	LONG	get_byte_immediate + (sbca << 9)                            'SBCA      Immediate , 2,    2, uaaaa
{83}	LONG	get_word_immediate + (subd << 9)                            'SUBD      Immediate , 4,    3, -aaaa
{84}	LONG	get_byte_immediate + (anda << 9)                            '*ANDA      Immediate , 2,    2, -aa0-
{85}	LONG	get_byte_immediate + (bita << 9)                            'BITA      Immediate , 2,    2, -aa0-
{86}	LONG	get_byte_immediate + (put_byte_a << 9)	                    '*LDA       Immediate , 2,    2, -aa0-
{87}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{88}	LONG	get_byte_immediate + (eora << 9)                            '*EORA      Immediate , 2,    2, -aa0-
{89}	LONG	get_byte_immediate + (adca << 9)                            'ADCA      Immediate , 2,    2, aaaaa
{8A}	LONG	get_byte_immediate + (ora  << 9)                            '*ORA       Immediate , 2,    2, -aa0-
{8B}	LONG	get_byte_immediate + (adda << 9)                            '*ADDA      Immediate , 2,    2, aaaaa
{8C}	LONG	get_word_immediate + (cmpx_<< 9)                            'CMPX      Immediate , 4,    3, -aaaa
{8D}	LONG	0	'BSR       Relative  , 7,    2, -----
{8E}	LONG	get_word_immediate + (put_word_x << 9)                      '*LDX       Immediate , 3,    3, -aa0-
{8F}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu

{90}	LONG	get_byte_direct    + (suba << 9)                            'SUBA      Direct    , 4,    2, uaaaa
{91}	LONG	get_byte_direct    + (cmpa << 9)                            'CMPA      Direct    , 4,    2, uaaaa
{92}	LONG	get_byte_direct    + (sbca << 9)                            'SBCA      Direct    , 4,    2, uaaaa
{93}	LONG	get_word_direct    + (subd << 9)                            'SUBD      Direct    , 6,    2, -aaaa
{94}	LONG	get_byte_direct    + (anda << 9)                            'ANDA      Direct    , 4,    2, -aa0-
{95}	LONG	get_byte_direct    + (bita << 9)                            'BITA      Direct    , 4,    2, -aa0-
{96}	LONG	get_byte_direct    + (put_byte_a << 9)                      'LDA       Direct    , 4,    2, -aa0-
{97}	LONG	put_a_direct                                                'STA       Direct    , 4,    2, -aa0-
{98}	LONG	get_byte_direct    + (eora << 9)                            'EORA      Direct    , 4,    2, -aa0-
{99}	LONG	get_byte_direct    + (adca << 9)                            'ADCA      Direct    , 4,    2, aaaaa
{9A}	LONG	get_byte_direct    + (ora  << 9)                            'ORA       Direct    , 4,    2, -aa0-
{9B}	LONG	get_byte_direct    + (adda << 9)                            'ADDA      Direct    , 4,    2, aaaaa
{9C}	LONG	get_word_direct    + (cmpx_<< 9)                            'CMPX      Direct    , 6,    2, -aaaa
{9D}	LONG	0	'JSR       Direct    , 7,    2, -----
{9E}	LONG	get_word_direct    + (put_word_x << 9)                      'LDX       Direct    , 5,    2, -aa0-
{9F}	LONG	get_word_x	   + (put_word_direct << 9)                 'STX       Direct    , 5,    2, -aa0-

{A0}	LONG	get_byte_indexed   + (suba << 9)                            'SUBA      Indexed   , 4,    2, uaaaa
{A1}	LONG	get_byte_indexed   + (cmpa << 9)                            'CMPA      Indexed   , 4,    2, uaaaa
{A2}	LONG	get_byte_indexed   + (sbca << 9)                            'SBCA      Indexed   , 4,    2, uaaaa
{A3}	LONG	get_word_indexed   + (subd << 9)                            'SUBD      Indexed   , 6,    2, -aaaa
{A4}	LONG	get_byte_indexed   + (anda << 9)                            'ANDA      Indexed   , 4,    2, -aa0-
{A5}	LONG	get_byte_indexed   + (bita << 9)                            'BITA      Indexed   , 4,    2, -aa0-
{A6}	LONG	get_byte_indexed   + (put_byte_a << 9)                      'LDA       Indexed   , 4,    2, -aa0-
{A7}	LONG	get_byte_a	   + (put_byte_indexed << 9)                'STA       Indexed   , 4,    2, -aa0-
{A8}	LONG	get_byte_indexed   + (eora << 9)                            'EORA      Indexed   , 4,    2, -aa0-
{A9}	LONG	get_byte_indexed   + (adca << 9)                            'ADCA      Indexed   , 4,    2, aaaaa
{AA}	LONG	get_byte_indexed   + (ora  << 9)                            'ORA       Indexed   , 4,    2, -aa0-
{AB}	LONG	get_byte_indexed   + (adda << 9)                            'ADDA      Indexed   , 4,    2, aaaaa
{AC}	LONG	get_word_indexed   + (cmpx_<< 9)                            'CMPX      Indexed   , 6,    2, -aaaa
{AD}	LONG	0	'JSR       Indexed   , 7,    2, -----
{AE}	LONG	get_word_indexed   + (put_word_x << 9)                      'LDX       Indexed   , 5,    2, -aa0-
{AF}	LONG	get_word_x	   + (put_word_indexed << 9)                'STX       Indexed   , 5,    2, -aa0-

{B0}	LONG	get_byte_extended  + (suba << 9)                            'SUBA      Extended  , 5,    3, uaaaa
{B1}	LONG	get_byte_extended  + (cmpa << 9)                            'CMPA      Extended  , 5,    3, uaaaa
{B2}	LONG	get_byte_extended  + (sbca << 9)                            'SBCA      Extended  , 5,    3, uaaaa
{B3}	LONG	get_word_extended  + (subd << 9)                            'SUBD      Extended  , 7,    3, -aaaa
{B4}	LONG	get_byte_extended  + (anda << 9)                            'ANDA      Extended  , 5,    3, -aa0-
{B5}	LONG	get_byte_extended  + (bita << 9)                            'BITA      Extended  , 5,    3, -aa0-
{B6}	LONG	get_byte_extended  + (put_byte_a << 9)                      '*LDA       Extended  , 5,    3, -aa0-
{B7}	LONG	get_byte_a	   + (put_byte_extended << 9)               'STA       Extended  , 5,    3, -aa0-
{B8}	LONG	get_byte_extended  + (eora << 9)                            'EORA      Extended  , 5,    3, -aa0-
{B9}	LONG	get_byte_extended  + (adca << 9)                            'ADCA      Extended  , 5,    3, aaaaa
{BA}	LONG	get_byte_extended  + (ora  << 9)                            'ORA       Extended  , 5,    3, -aa0-
{BB}	LONG	get_byte_extended  + (adda << 9)                            'ADDA      Extended  , 5,    3, aaaaa
{BC}	LONG	get_word_extended  + (cmpx_<< 9)                            'CMPX      Extended  , 7,    3, -aaaa
{BD}	LONG	0	'JSR       Extended  , 8,    3, -----
{BE}	LONG	get_word_extended  + (put_word_x << 9)                      'LDX       Extended  , 6,    3, -aa0-
{BF}	LONG	get_word_x	   + (put_word_extended << 9)               'STX       Extended  , 6,    3, -aa0-

{C0}	LONG	get_byte_immediate + (subb << 9)                            '*SUBB      Immediate , 2,    2, uaaaa
{C1}	LONG	get_byte_immediate + (cmpb << 9)	                    'CMPB      Immediate , 2,    2, uaaaa
{C2}	LONG	get_byte_immediate + (sbcb << 9)                            'SBCB      Immediate , 2,    2, uaaaa
{C3}	LONG	get_word_immediate + (addd << 9)                            '*ADDD      Immediate , 4,    3, -aaaa
{C4}	LONG	get_byte_immediate + (andb << 9)                            '*ANDB      Immediate , 2,    2, -aa0-
{C5}	LONG	get_byte_immediate + (bitb << 9)                            'BITB      Immediate , 2,    2, -aa0-
{C6}	LONG	get_byte_immediate + (put_byte_b << 9)                      '*LDB       Immediate , 2,    2, -aa0-
{C7}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{C8}	LONG	get_byte_immediate + (eorb << 9)                            '*EORB      Immediate , 2,    2, -aa0-
{C9}	LONG	get_byte_immediate + (adcb << 9)                            'ADCB      Immediate , 2,    2, aaaaa
{CA}	LONG	get_byte_immediate + (orb  << 9)                            '*ORB       Immediate , 2,    2, -aa0-
{CB}	LONG	get_byte_immediate + (addb << 9)                            '*ADDB      Immediate , 2,    2, aaaaa
{CC}	LONG	get_word_immediate + (put_word_d << 9)                      '*LDD       Immediate , 3,    3, -aa0-
{CD}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu
{CE}	LONG	get_word_immediate + (put_word_u << 9)                      '*LDU       Immediate , 3,    3, -aa0-
{CF}	LONG	illegal	                                                    'Illegal             , 1,    1, uuuuu

{D0}	LONG	get_byte_direct    + (subb << 9)                            'SUBB      Direct    , 4,    2, uaaaa
{D1}	LONG	get_byte_direct    + (cmpb << 9)                            'CMPB      Direct    , 4,    2, uaaaa
{D2}	LONG	get_byte_direct    + (sbcb << 9)                            'SBCB      Direct    , 4,    2, uaaaa
{D3}	LONG	get_byte_direct    + (addd << 9)                            'ADDD      Direct    , 6,    2, -aaaa
{D4}	LONG	get_byte_direct    + (andb << 9)                            'ANDB      Direct    , 4,    2, -aa0-
{D5}	LONG	get_byte_direct    + (bitb << 9)                            'BITB      Direct    , 4,    2, -aa0-
{D6}	LONG	get_byte_direct    + (put_byte_b << 9)                      'LDB       Direct    , 4,    2, -aa0-
{D7}	LONG	put_b_direct                                                'STB       Direct    , 4,    2, -aa0-
{D8}	LONG	get_byte_direct    + (eorb << 9)                            'EORB      Direct    , 4,    2, -aa0-
{D9}	LONG	get_byte_direct    + (adcb << 9)                            'ADCB      Direct    , 4,    2, aaaaa
{DA}	LONG	get_byte_direct    + (orb  << 9)                            'ORB       Direct    , 4,    2, -aa0-
{DB}	LONG	get_byte_direct    + (addb << 9)                            'ADDB      Direct    , 4,    2, aaaaa
{DC}	LONG	get_word_direct    + (put_word_d << 9)                      'LDD       Direct    , 5,    2, -aa0-
{DD}	LONG	put_d_direct                                                'STD       Direct    , 5,    2, -aa0-
{DE}	LONG	get_word_direct    + (put_word_u << 9)                      'LDU       Direct    , 5,    2, -aa0-
{DF}	LONG	put_u_direct                                                'STU       Direct    , 5,    2, -aa0-

{E0}	LONG	get_byte_indexed   + (subb << 9)                            'SUBB      Indexed   , 4,    2, uaaaa
{E1}	LONG	get_byte_indexed   + (cmpb << 9)                            'CMPB      Indexed   , 4,    2, uaaaa
{E2}	LONG	get_byte_indexed   + (sbcb << 9)                            'SBCB      Indexed   , 4,    2, uaaaa
{E3}	LONG	get_word_indexed   + (addd << 9)                            'ADDD      Indexed   , 6,    2, -aaaa
{E4}	LONG	get_byte_indexed   + (andb << 9)                            'ANDB      Indexed   , 4,    2, -aa0-
{E5}	LONG	get_byte_indexed   + (bitb << 9)                            'BITB      Indexed   , 4,    2, -aa0-
{E6}	LONG	get_byte_indexed   + (put_byte_b << 9)                      'LDB       Indexed   , 4,    2, -aa0-
{E7}	LONG	get_byte_b	   + (put_word_indexed << 9)                'STB       Indexed   , 4,    2, -aa0-
{E8}	LONG	get_byte_indexed   + (eorb << 9)                            'EORB      Indexed   , 4,    2, -aa0-
{E9}	LONG	get_byte_indexed   + (adcb << 9)                            'ADCB      Indexed   , 4,    2, aaaaa
{EA}	LONG	get_byte_indexed   + (orb  << 9)                            'ORB       Indexed   , 4,    2, -aa0-
{EB}	LONG	get_byte_indexed   + (addb << 9)                            'ADDB      Indexed   , 4,    2, aaaaa
{EC}	LONG	get_word_indexed   + (put_word_d << 9)                      'LDD       Indexed   , 5,    2, -aa0-
{ED}	LONG	get_word_d	   + (put_word_indexed << 9)                'STD       Indexed   , 5,    2, -aa0-
{EE}	LONG	get_word_indexed   + (put_word_u << 9)                      'LDU       Indexed   , 5,    2, -aa0-
{EF}	LONG	get_word_u	   + (put_word_indexed << 9)                'STU       Indexed   , 5,    2, -aa0-

{F0}	LONG	get_byte_extended  + (subb << 9)                            'SUBB      Extended  , 5,    3, uaaaa
{F1}	LONG	get_byte_extended  + (cmpb << 9)                            'CMPB      Extended  , 5,    3, uaaaa
{F2}	LONG	get_byte_extended  + (sbcb << 9)                            'SBCB      Extended  , 5,    3, uaaaa
{F3}	LONG	get_word_extended  + (addd << 9)                            'ADDD      Extended  , 7,    3, -aaaa
{F4}	LONG	get_byte_extended  + (andb << 9)                            'ANDB      Extended  , 5,    3, -aa0-
{F5}	LONG	get_byte_extended  + (bitb << 9)                            'BITB      Extended  , 5,    3, -aa0-
{F6}	LONG	get_byte_extended  + (put_byte_b << 9)                      'LDB       Extended  , 5,    3, -aa0-
{F7}	LONG	get_byte_b	   + (put_byte_extended << 9)               'STB       Extended  , 5,    3, -aa0-
{F8}	LONG	get_byte_extended  + (eorb << 9)                            'EORB      Extended  , 5,    3, -aa0-
{F9}	LONG	get_byte_extended  + (adcb << 9)                            'ADCB      Extended  , 5,    3, aaaaa
{FA}	LONG	get_byte_extended  + (orb  << 9)                            'ORB       Extended  , 5,    3, -aa0-
{FB}	LONG	get_byte_extended  + (addb << 9)                            'ADDB      Extended  , 5,    3, aaaaa
{FC}	LONG	get_word_extended  + (put_word_d << 9)                      'LDD       Extended  , 6,    3, -aa0-
{FD}	LONG	get_word_d	   + (put_word_extended << 9)               'STD       Extended  , 6,    3, -aa0-
{FE}	LONG	get_word_extended  + (put_word_u << 9)                      'LDU       Extended  , 6,    3, -aa0-
{FF}	LONG	get_word_u	   + (put_word_extended << 9)               'STU       Extended  , 6,    3, -aa0-
'---------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------
dispatch_table_1
{00}	LONG	0	'Illegal
{01}	LONG	0	'Illegal
{02}	LONG	0	'Illegal
{03}	LONG	0	'Illegal
{04}	LONG	0	'Illegal
{05}	LONG	0	'Illegal
{06}	LONG	0	'Illegal
{07}	LONG	0	'Illegal
{08}	LONG	0	'Illegal
{09}	LONG	0	'Illegal
{0A}	LONG	0	'Illegal
{0B}	LONG	0	'Illegal
{0C}	LONG	0	'Illegal
{0D}	LONG	0	'Illegal
{0E}	LONG	0	'Illegal
{0F}	LONG	0	'Illegal

{10}	LONG	0	'Illegal
{11}	LONG	0	'Illegal
{12}	LONG	0	'Illegal
{13}	LONG	0	'Illegal
{14}	LONG	0	'Illegal
{15}	LONG	0	'Illegal
{16}	LONG	0	'Illegal
{17}	LONG	0	'Illegal
{18}	LONG	0	'Illegal
{19}	LONG	0	'Illegal
{1A}	LONG	0	'Illegal
{1B}	LONG	0	'Illegal
{1C}	LONG	0	'Illegal
{1D}	LONG	0	'Illegal
{1E}	LONG	0	'Illegal
{1F}	LONG	0	'Illegal

{20}	LONG	0	'Illegal
{21}	LONG	0	'LBRN      Relative  , 5(6), 4, -----
{22}	LONG	0	'LBHI      Relative  , 5(6), 4, -----
{23}	LONG	0	'LBLS      Relative  , 5(6), 4, -----
{24}	LONG	0	'LBHS/LBCC Relative  , 5(6), 4, -----
{25}	LONG	0	'LBLO/LBCS Relative  , 5(6), 4, -----
{26}	LONG	0	'LBNE      Relative  , 5(6), 4, -----
{27}	LONG	0	'LBEQ      Relative  , 5(6), 4, -----
{28}	LONG	0	'LBVC      Relative  , 5(6), 4, ----- 
{29}	LONG	0	'LBVS      Relative  , 5(6), 4, -----
{2A}	LONG	0	'LBPL      Relative  , 5(6), 4, -----
{2B}	LONG	0	'LBMI      Relative  , 5(6), 4, -----
{2C}	LONG	0	'LBGE      Relative  , 5(6), 4, -----
{2D}	LONG	0	'LBLT      Relative  , 5(6), 4, -----
{2E}	LONG	0	'LBGT      Relative  , 5(6), 4, -----
{2F}	LONG	0	'LBLE      Relative  , 5(6), 4, -----

{30}	LONG	0	'Illegal
{31}	LONG	0	'Illegal
{32}	LONG	0	'Illegal
{33}	LONG	0	'Illegal
{34}	LONG	0	'Illegal
{35}	LONG	0	'Illegal
{36}	LONG	0	'Illegal
{37}	LONG	0	'Illegal
{38}	LONG	0	'Illegal
{39}	LONG	0	'Illegal
{3A}	LONG	0	'Illegal
{3B}	LONG	0	'Illegal
{3C}	LONG	0	'Illegal
{3D}	LONG	0	'Illegal
{3E}	LONG	0	'Illegal
{3F}	LONG	0	'SWI2      Inherent  , 20,   2, -----

{40}	LONG	0	'Illegal
{41}	LONG	0	'Illegal
{42}	LONG	0	'Illegal
{43}	LONG	0	'Illegal
{44}	LONG	0	'Illegal
{45}	LONG	0	'Illegal
{46}	LONG	0	'Illegal
{47}	LONG	0	'Illegal
{48}	LONG	0	'Illegal
{49}	LONG	0	'Illegal
{4A}	LONG	0	'Illegal
{4B}	LONG	0	'Illegal
{4C}	LONG	0	'Illegal
{4D}	LONG	0	'Illegal
{4E}	LONG	0	'Illegal
{4F}	LONG	0	'Illegal

{50}	LONG	0	'Illegal
{51}	LONG	0	'Illegal
{52}	LONG	0	'Illegal
{53}	LONG	0	'Illegal
{54}	LONG	0	'Illegal
{55}	LONG	0	'Illegal
{56}	LONG	0	'Illegal
{57}	LONG	0	'Illegal
{58}	LONG	0	'Illegal
{59}	LONG	0	'Illegal
{5A}	LONG	0	'Illegal
{5B}	LONG	0	'Illegal
{5C}	LONG	0	'Illegal
{5D}	LONG	0	'Illegal
{5E}	LONG	0	'Illegal
{5F}	LONG	0	'Illegal

{60}	LONG	0	'Illegal
{61}	LONG	0	'Illegal
{62}	LONG	0	'Illegal
{63}	LONG	0	'Illegal
{64}	LONG	0	'Illegal
{65}	LONG	0	'Illegal
{66}	LONG	0	'Illegal
{67}	LONG	0	'Illegal
{68}	LONG	0	'Illegal
{69}	LONG	0	'Illegal
{6A}	LONG	0	'Illegal
{6B}	LONG	0	'Illegal
{6C}	LONG	0	'Illegal
{6D}	LONG	0	'Illegal
{6E}	LONG	0	'Illegal
{6F}	LONG	0	'Illegal

{70}	LONG	0	'Illegal
{71}	LONG	0	'Illegal
{72}	LONG	0	'Illegal
{73}	LONG	0	'Illegal
{74}	LONG	0	'Illegal
{75}	LONG	0	'Illegal
{76}	LONG	0	'Illegal
{77}	LONG	0	'Illegal
{78}	LONG	0	'Illegal
{79}	LONG	0	'Illegal
{7A}	LONG	0	'Illegal
{7B}	LONG	0	'Illegal
{7C}	LONG	0	'Illegal
{7D}	LONG	0	'Illegal
{7E}	LONG	0	'Illegal
{7F}	LONG	0	'Illegal

{80}	LONG	0	'Illegal
{81}	LONG	0	'Illegal
{82}	LONG	0	'Illegal
{83}	LONG	0	'CMPD      Imediate  , 5,    4, -aaaa
{84}	LONG	0	'Illegal
{85}	LONG	0	'Illegal
{86}	LONG	0	'Illegal
{87}	LONG	0	'Illegal
{88}	LONG	0	'Illegal
{89}	LONG	0	'Illegal
{8A}	LONG	0	'Illegal
{8B}	LONG	0	'Illegal
{8C}	LONG	0	'CMPY      Imediate  , 5,    4, -aaaa
{8D}	LONG	0	'Illegal
{8E}	LONG	get_word_immediate + (put_word_y << 9)     'LDY       Imediate  , 4,    4, -aa0-
{8F}	LONG	0	'Illegal

{90}	LONG	0	'Illegal
{91}	LONG	0	'Illegal
{92}	LONG	0	'Illegal
{93}	LONG	0	'CMPD      Direct    , 7,    3, -aaaa
{94}	LONG	0	'Illegal
{95}	LONG	0	'Illegal
{96}	LONG	0	'Illegal
{97}	LONG	0	'Illegal
{08}	LONG	0	'Illegal
{99}	LONG	0	'Illegal
{9A}	LONG	0	'Illegal
{9B}	LONG	0	'Illegal
{9C}	LONG	0	'CMPY      Direct    , 7,    3, -aaaa
{9D}	LONG	0	'Illegal
{9E}	LONG	get_word_direct + (put_word_y << 9)      'LDY       Direct    , 6,    3, -aa0-
{9F}	LONG	get_word_x	+ (put_word_direct << 9)  'STY       Direct    , 6,    3, -aa0-

{A0}	LONG	0	'Illegal
{A1}	LONG	0	'Illegal
{A2}	LONG	0	'Illegal
{A3}	LONG	0	'CMPD      Indexed   , 7,    3, -aaaa
{A4}	LONG	0	'Illegal
{A5}	LONG	0	'Illegal
{A6}	LONG	0	'Illegal
{A7}	LONG	0	'Illegal
{A8}	LONG	0	'Illegal
{A9}	LONG	0	'Illegal
{AA}	LONG	0	'Illegal
{AB}	LONG	0	'Illegal
{AC}	LONG	0	'CMPY      Indexed   , 7,    3, -aaaa
{AD}	LONG	0	'Illegal
{AE}	LONG	get_word_indexed + (put_word_y << 9)     'LDY       Indexed   , 6,    3, -aa0-
{AF}	LONG	get_word_y	+ (put_word_indexed << 9)'STY       Indexed   , 6,    3, -aa0-

{B0}	LONG	0	'Illegal
{B1}	LONG	0	'Illegal
{B2}	LONG	0	'Illegal
{B3}	LONG	0	'CMPD      Extended  , 8,    4, -aaaa
{B4}	LONG	0	'Illegal
{B5}	LONG	0	'Illegal
{B6}	LONG	0	'Illegal
{B7}	LONG	0	'Illegal
{B8}	LONG	0	'Illegal
{B9}	LONG	0	'Illegal
{BA}	LONG	0	'Illegal
{BB}	LONG	0	'Illegal
{BC}	LONG	0	'CMPY      Extended  , 8,    4, -aaaa
{BD}	LONG	0	'Illegal
{BE}	LONG	get_word_extended + (put_word_y << 9)       'LDY       Extended  , 7,    4, -aa0-
{BF}	LONG	get_word_y	  + (put_word_extended << 9)'STY       Extended  , 7,    4, -aa0-

{C0}	LONG	0	'Illegal
{C1}	LONG	0	'Illegal
{C2}	LONG	0	'Illegal
{C3}	LONG	0	'Illegal
{C4}	LONG	0	'Illegal
{C5}	LONG	0	'Illegal
{C6}	LONG	0	'Illegal
{C7}	LONG	0	'Illegal
{C8}	LONG	0	'Illegal
{C9}	LONG	0	'Illegal
{CA}	LONG	0	'Illegal
{CB}	LONG	0	'Illegal
{CD}	LONG	0	'Illegal
{CE}	LONG	get_word_immediate + (put_word_s << 9)    'LDS       Imediate  , 4,    4, -aa0-
{CF}	LONG	0	'Illegal

{D0}	LONG	0	'Illegal
{D1}	LONG	0	'Illegal
{D2}	LONG	0	'Illegal
{D3}	LONG	0	'Illegal
{D4}	LONG	0	'Illegal
{D5}	LONG	0	'Illegal
{D6}	LONG	0	'Illegal
{D7}	LONG	0	'Illegal
{D8}	LONG	0	'Illegal
{D9}	LONG	0	'Illegal
{DA}	LONG	0	'Illegal
{DB}	LONG	0	'Illegal
{DC}	LONG	0	'Illegal
{DD}	LONG	0	'Illegal
{DE}	LONG	get_word_direct + (put_word_s << 9)       'LDS       Direct    , 6,    3, -aa0-
{DF}	LONG	get_word_s	+ (put_word_direct << 9)  'STS       Direct    , 6,    3, -aa0-

{E0}	LONG	0	'Illegal
{E1}	LONG	0	'Illegal
{E2}	LONG	0	'Illegal
{E3}	LONG	0	'Illegal
{E4}	LONG	0	'Illegal
{E5}	LONG	0	'Illegal
{E6}	LONG	0	'Illegal
{E7}	LONG	0	'Illegal
{E8}	LONG	0	'Illegal
{E9}	LONG	0	'Illegal
{EA}	LONG	0	'Illegal
{EB}	LONG	0	'Illegal
{EC}	LONG	0	'Illegal
{ED}	LONG	0	'Illegal
{EE}	LONG	get_word_indexed + (put_word_s << 9)       'LDS       Indexed   , 6,    3, -aa0-
{EF}	LONG	get_word_s	 + (put_word_indexed << 9) 'STS       Indexed   , 6,    3, -aa0-

{F0}	LONG	0	'Illegal
{F1}	LONG	0	'Illegal
{F2}	LONG	0	'Illegal
{F3}	LONG	0	'Illegal
{F4}	LONG	0	'Illegal
{F5}	LONG	0	'Illegal
{F6}	LONG	0	'Illegal
{F7}	LONG	0	'Illegal
{F8}	LONG	0	'Illegal
{F9}	LONG	0	'Illegal
{FA}	LONG	0	'Illegal
{FB}	LONG	0	'Illegal
{FC}	LONG	0	'Illegal
{FD}	LONG	0	'Illegal
{FE}	LONG	get_word_extended + (put_word_s << 9)       'LDS       Extended  , 7,    4, -aa0-
{FF}	LONG	get_word_s	 + (put_word_extended << 9) 'STS       Extended  , 7,    4, -aa0-
'---------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------
dispatch_table_2
{113F}	LONG	0	'SWI3      Inherent  ,20,    2, -----
{1183}	LONG	0	'CMPU      Imediate  , 5,    4, -aaaa
{118C}	LONG	0	'CMPS      Imediate  , 5,    4, -aaaa
{1193}	LONG	0	'CMPU      Direct    , 7,    3, -aaaa
{119C}	LONG	0	'CMPS      Direct    , 7,    3, -aaaa
{11A3}	LONG	0	'CMPU      Indexed   , 7,    3, -aaaa
{11AC}	LONG	0	'CMPS      Indexed   , 7,    3, -aaaa
{11B3}	LONG	0	'CMPU      Extended  , 8,    4, -aaaa
{11BC}	LONG	0	'CMPS      Extended  , 8,    4, -aaaa
'---------------------------------------------------------------------------------------------------


index_postbyte_table
{00}    LONG  index_post_inc     ',R+
{01}    LONG  index_post_inc_2   ',R++
{02}    LONG  index_pre_dec      ',-R
{03}    LONG  index_pre_dec_2    ',--R
{04}    LONG  index              ',R
{05}    LONG  index_b            '(+/- B),R
{06}    LONG  index_a            '(+/- A),R
{07}    LONG  illegal            'Illegal
{08}    LONG  index_offset_8     '(+/- 7 bit offset),R
{09}    LONG  index_offset_16    '(+/- 15 bit offset),R
{0A}    LONG  illegal            'Illegal
{0B}    LONG  index_d            '(+/- D),R
{0C}    LONG  pc_offset_8        '(+/- 7 bit offset),PC
{0D}    LONG  pc_offset_16       '(+/- 15 bit offset),PC
{0E}    LONG  illegal            'Illegal
{0F}    LONG  extended_indirect  '[address]

' The end.
'---------------------------------------------------------------------------------------------------


